
; Driver for 3W Luxeon Led with MCP1652 and PIC10F204
; Use 6 steps for setting contrast - 10%,25%,50%,75%;100%, slow flash and fast flash
; switch off is after hold on 3 sec. switch on/off/mode
; low voltage baterii is indicate with red led on the MCP1652 output
; for 4x NIMH bateriis is 3 volt setting for indicate batery deplete
; substance ware based on www.microchip.com - DS 51513A
; Pavel Reznicek Ceska Lipa Ver.1.0 www.elektronikacl.wz.cz

; Constants
IRATE		EQU		d'255' - d'124'		; TMR0 reset value for ~ 1000Hz
LED		EQU		d'0'			; GPIO bit for LED control
BUTTON		EQU		d'1'			; GPIO bit for button

; Variables
		cblock	0x10				; Beginning of address space
		tcnt					; Timer counter (to divide down counter)
		pwmcnt:2				; PWM counter
		pwmper:2				; PWM period
		pwmduty:2				; PWM duty cycle

;		democnt					; Demo time counter (time between mode changes)
		mode					; LED Operating mode
;		demo					; Boolean to indicate is demo operation
		bcnt					; Button press duration counter
		tmp					; Universal registr 
		tmp1					; Universal registr 1
		freg					; Flag registr 
		endc

;***************************************************************************
		LIST   P=10F204
                #include P10F204.INC
	__CONFIG _CP_ON & _WDT_OFF & _MCLRE_OFF 
;***************************************************************************

		ORG	0x00

Reset		movwf	OSCCAL			; Calibrate the oscillator
		bcf	OSCCAL,0
		movlw	b'11110011'		; Disable the comparator
		movwf	CMCON0
		movlw	b'11000010'		; Setup TMR0 with 1:8 prescale,Pull-up off, Wake up from Sleep - off
		option				; using internal clock
		movlw	IRATE			; Set TMR0 count
		movwf	TMR0
		movlw	b'00000010'		; Set DDR bits
		tris	GPIO

Test_Reset	movlw	b'10010000'
		andwf	STATUS,w
		xorlw	B'10010000'
		skpnz				;goto IncMode
		goto	IncMode			;mode+1

;clear all user registrs
		movlw	0x10			;first address ram registr
		movwf	FSR
Clear_Ram	clrf	INDF
		incf	FSR,f
		btfsc	FSR,4			;is 20h ram?
		goto	Clear_Ram
		clrf	GPIO
		goto	OFF			; go to baterii OFF

Main		movf	TMR0,W			; Has the timer rolled?
		btfss	STATUS,Z
		goto	Main
		movlw	IRATE			; Reset TMR0
		movwf	TMR0

		incf	pwmcnt,F		; Increment PWM counter
		btfsc	STATUS,Z
		incf	pwmcnt+1,F
		movf	pwmper,W		; Compare period to counter
		subwf	pwmcnt,W
		btfss	STATUS,Z		; Low byte != zero
		goto	Duty
		movf	pwmper+1,W
		subwf	pwmcnt+1,W
		btfss	STATUS,Z		; High byte != zero
		goto	Duty
		clrf	pwmcnt			; Clear the PWM counter
		clrf	pwmcnt+1
		bsf	GPIO,LED		; Turn on the LED
Duty		movf	pwmduty,W
		subwf	pwmcnt,W
		btfss	STATUS,Z		; Low byte != zero
		goto	Button
		movf	pwmduty+1,W
		subwf	pwmcnt+1,W
		btfss	STATUS,Z		; High byte != zero
		goto	Button
		bcf	GPIO,LED		; Turn off the LED

Button		decfsz	tcnt,F			; Divide down button counter by 25
		goto	Main			; This is done so that everything else
		movlw	d'25'			; fits into one byte
		movwf	tcnt

; switch on/mode/off  - aproxi. after 3 sek you are going to turn off lamp

		btfss	GPIO,BUTTON		; Is the button pressed
		goto	Down

		movlw	D'75'
		subwf	bcnt,w
		skpc
		clrf	bcnt
				
		bcf	freg,0			; Reset flag
		clrf	bcnt
		goto	Main


Down		movlw	d'75'			; is 3 seconds?
		subwf	bcnt,W
		btfsc	STATUS,Z
		goto	OFF			; after 3 second go to sleep  - led will be switch off
		incf	bcnt,F			; Increment the button timer counter
	
Down_1		btfsc	freg,0			; probe flag
		goto	Main
		bsf	freg,0			; set flag

IncMode		incf	mode,F			; Go to the next mode
		movlw	0x08			; max programs
		subwf	mode,w
		skpz
		goto	SetMode
		movlw	0x01
		movwf	mode

SetMode		clrf	pwmcnt			; Clear the PWM counter
		clrf	pwmcnt+1
		bsf	GPIO,LED		; Set the LED on
		movlw	b'00000111'
		andwf	mode,W
		addwf	PCL,F
		nop
		goto	Mode			; 10% brightness
		goto	Mode0			; 25% brightness
		goto	Mode1			; 50% brightness
		goto	Mode2			; 75% brightness
		goto	Mode3			; Continuously ON
		goto	Mode4			; Slow flash rate
		goto	Mode5			; Fast flash rate

Mode		movlw	LOW(d'10')		; 10% brightness
		movwf	pwmper
		movlw	HIGH(d'10')
		movwf	pwmper+1
		movlw	LOW(d'1')
		movwf	pwmduty
		movlw	HIGH(d'1')
		movwf	pwmduty+1
		goto	Main

Mode0		movlw	LOW(d'10')		; 25% brightness
		movwf	pwmper
		movlw	HIGH(d'10')
		movwf	pwmper+1
		movlw	LOW(d'2')
		movwf	pwmduty
		movlw	HIGH(d'2')
		movwf	pwmduty+1
		goto	Main
	

Mode1		movlw	LOW(d'10')		; 50% brightness
		movwf	pwmper
		movlw	HIGH(d'10')
		movwf	pwmper+1
		movlw	LOW(d'5')
		movwf	pwmduty
		movlw	HIGH(d'5')
		movwf	pwmduty+1
		goto	Main


Mode2		movlw	LOW(d'10')		; 75% brightness
		movwf	pwmper
		movlw	HIGH(d'10')
		movwf	pwmper+1
		movlw	LOW(d'7')
		movwf	pwmduty
		movlw	HIGH(d'7')
		movwf	pwmduty+1
		goto	Main


Mode3		movlw	LOW(d'100')		; Continuously ON - 100% brighitess
		movwf	pwmper
		movlw	HIGH(d'100')
		movwf	pwmper+1
		movlw	LOW(d'101')
		movwf	pwmduty
		movlw	HIGH(d'101')
		movwf	pwmduty+1
		goto	Main


Mode4		movlw	LOW(d'200')		; Slow flash rate
		movwf	pwmper
		movlw	HIGH(d'200')
		movwf	pwmper+1
		movlw	LOW(d'100')
		movwf	pwmduty
		movlw	HIGH(d'100')
		movwf	pwmduty+1
		goto	Main

Mode5		movlw	LOW(d'100')		; Fast flash rate
		movwf	pwmper
		movlw	HIGH(d'100')
		movwf	pwmper+1
		movlw	LOW(d'50')
		movwf	pwmduty
		movlw	HIGH(d'50')
		movwf	pwmduty+1
		goto	Main

OFF		bcf	GPIO,LED
		clrf	bcnt
		btfss	GPIO,BUTTON		; wait for enable switch
		goto	$-1
		call	Wait			;
		call	Wait
		clrf	mode
;		clrf	freg
		movlw	b'01000010'		; Setup TMR0 with 1:8 prescale,Pull-up on, Wake up from Sleep - on
		option				; using internal clock
		sleep				; Goto sleep

Wait		movlw	0xFF			; wait 180mS
		movwf	tmp1			; 
		movlw	0xFF
		movwf	tmp
		decfsz	tmp,f
		goto	$-1
		decfsz	tmp1,f
		goto	$-5
		retlw	0

		END
